/**
  **************************************************************************
  * @file     at32_video_ev_spi.h
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_spi header file
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */
  
#ifndef __AT32_VIDEO_EV_SPI_H
#define __AT32_VIDEO_EV_SPI_H
#include "at32f403a_407_board.h"
#include "at32_video_ev_lcd.h"
/***************** GPIO  config *********************/

#define SPIRxBufferSize		               (IMAGE_WIDTH*2)
#define DMARxSize 			                 (IMAGE_WIDTH*2 + 6)
#define LCDSPIDMATxSize   	             (IMAGE_WIDTH*2)
#define RxXyySize 			                 (400)

#define VsyncSize 			                 (9)
#define HsyncSize 			                 (6)

#define  BF3901_SPI_PORT						     3
#define  BF3901_SPI_SELECTED					   SPI2
#define  BF3901_SPI_CLK							     CRM_SPI2_PERIPH_CLOCK
#define  BF3901_SPI_GPIO						     GPIOB
#define  BF3901_SPI_GPIO_CLK					   CRM_GPIOB_PERIPH_CLOCK 
#define  BF3901_SPI_PIN_SCK						   GPIO_PINS_13
#define  BF3901_SPI_PIN_MISO					   GPIO_PINS_14
#define  BF3901_SPI_PIN_MOSI					   GPIO_PINS_15 
#define  BF3901_SPI_IRQn						     SPI1_IRQn
#define  BF3901_SPI_SCK 							   PBin(13)
#define  BF3901_SPI_MISO						     PBin(14)

#define BF3901_SPI_SLAVE_DMA             DMA1
#define BF3901_SPI_SLAVE_DMA_CLK         CRM_DMA1_PERIPH_CLOCK
#define BF3901_SPI_SLAVE_Rx_DMA_IRQn     DMA1_Channel4_IRQn
#define BF3901_SPI_SLAVE_Rx_DMA_Channel  DMA1_CHANNEL4
#define BF3901_SPI_SLAVE_Rx_DMA_INT      DMA1_FDT4_FLAG
#define BF3901_SPI_SLAVE_Rx_DMA_FLAG     DMA1_FDT4_FLAG

#define BF3901_SPI_SLAVE_DR_Base         (uint32_t)(&(SPI2->dt)) 

#define  LCD_SPI_SELECTED				 	       SPI1
#define  LCD_SPI_SELECTED_CLK				     CRM_SPI1_PERIPH_CLOCK
#define  LCD_SPI_SELECTED_GPIO				   GPIOA
#define  LCD_SPI_SELECTED_GPIO_CLK			 CRM_GPIOA_PERIPH_CLOCK 
#define  LCD_SPI_SELECTED_PIN_NSS			   GPIO_PINS_4
#define  LCD_SPI_SELECTED_PIN_SCK			   GPIO_PINS_5
#define  LCD_SPI_SELECTED_PIN_MISO			 GPIO_PINS_6
#define  LCD_SPI_SELECTED_PIN_MOSI			 GPIO_PINS_7 
#define  LCD_SPI_SELECTED_IRQn				   SPI1_IRQn
#define  LCD_SPI_SCK 						         PAin(5)
#define  LCD_SPI_MISO						         PAin(6)

#define LCD_SPI_MASTER_DMA               DMA1
#define LCD_SPI_MASTER_DMA_CLK           CRM_DMA1_PERIPH_CLOCK
#define LCD_SPI_MASTER_Tx_DMA_IRQn       DMA1_Channel3_IRQn
#define LCD_SPI_MASTER_Rx_DMA_Channel    DMA1_CHANNEL2
#define LCD_SPI_MASTER_Rx_DMA_INT        DMA1_FDT2_FLAG
#define LCD_SPI_MASTER_Rx_DMA_FLAG       DMA1_FDT2_FLAG
#define LCD_SPI_MASTER_Tx_DMA_Channel    DMA1_CHANNEL3
#define LCD_SPI_MASTER_Tx_DMA_INT        DMA1_FDT3_FLAG
#define LCD_SPI_MASTER_Tx_DMA_FLAG       DMA1_FDT3_FLAG
#define LCD_SPI_MASTER_DR_Base			     (uint32_t)(&(SPI1->dt)); 

extern __IO u8  SPITxEndFlg;

void bf3901_spi_init(void);
void lcd_spi1_init(void);
void lcd_spi1_senddata(u8 txdata);
void lcd_spi1_sendindex(u8 txdata);
u8 lcd_spi1_receivedata(void);

#endif

