/*
 **************************************************************************
 * File Name    : hw_config.c
 * Description  : Hardware Configuration & setup
 * Date         : 2016-12-14
 * Version      : V1.0.0
 **************************************************************************
 */


/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
#include "usb_lib.h"
#include "usb_desc.h"
#include "usb_pwr.h"
#include "string.h"	
#include "stdarg.h"		 
#include "stdio.h"	

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
usb_usart_fifo usb_txfifo;
usb_usart_fifo usb_rxfifo;


uint8_t usb_packet_sent = 0;
uint8_t usb_packet_receive = 0;
uint8_t usb_Receive_Buffer[64];
uint16_t usb_Receive_length;

u8  USART_PRINTF_Buffer[USB_USART_REC_LEN];	//usb_printfͻ

static void IntToUnicode (uint32_t value , uint8_t *pbuf , uint8_t len);
 
/* Extern variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/**
  * @brief  Set USB Prescaler
  *         Set and cleared by software to generate 48MHz USB Clock 
  * @param  None
  * @retval None
  */
void Set_USBClock(unsigned char bDivByPRE)
{
    switch (SystemCoreClock)
    {
        case 48000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div1);
            break;
        case 72000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div1_5);
            break;
        case 96000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div2);
            break;
        case 120000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div2_5);
            break;
        case 144000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div3);
            break;
        case 168000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div3_5);
            break;
        case 192000000:
            RCC_USBCLKConfig(RCC_USBCLKSelection_PLL_Div4);
            break;
        default:
            break;
    }
    RCC_APB1PeriphClockCmd(RCC_APB1PERIPH_USB, ENABLE);
				 
} 
 
/*******************************************************************************
* Function Name  : Enter_LowPowerMode.
* Description    : Power-off system clocks and power while entering suspend mode.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void Enter_LowPowerMode(void)
{
  /* Set the device state to suspend */
  bDeviceState = SUSPENDED;	  
  /* Request to enter STOP mode with regulator in low power mode */
//  PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFI);
}			  

/*******************************************************************************
* Function Name  : Leave_LowPowerMode.
* Description    : Restores system clocks and power while exiting suspend mode.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void Leave_LowPowerMode(void)
{
  DEVICE_INFO *pInfo = &Device_Info;
								   
													   
  /* Set the device state to the correct state */
  if (pInfo->Current_Configuration != 0)
  {
    /* Device configured */
    bDeviceState = CONFIGURED;
		//printf("CONFIGURED\r\n");
  }
  else
  {
    bDeviceState = ATTACHED;
		//printf("ATTACHED\r\n");
  }
}

/**
  * @brief  Configure Interrupt
  * @param  None
  * @retval None
  */
void USB_Interrupts_Config(void)
{
    NVIC_InitType  NVIC_InitStructure;
    EXTI_InitType EXTI_InitStructure;
    
    /*Enable EXTI18 as interrupt Mode*/
    EXTI_ClearIntPendingBit(EXTI_Line18);
    EXTI_InitStructure.EXTI_Line = EXTI_Line18;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineEnable = ENABLE;
    EXTI_Init(&EXTI_InitStructure);
    
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
    
    /*Enable USB low priority interrupt*/
    NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    
    /*Enable USB WakeUP interrupt*/
    NVIC_InitStructure.NVIC_IRQChannel = USBWakeUp_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x2;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    	 	 	
}
		    
/*******************************************************************************
* Function Name  : Get_SerialNum.
* Description    : Create the serial number string descriptor.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void Get_SerialNum(void)
{
  u32 Device_Serial0, Device_Serial1, Device_Serial2;

  Device_Serial0 = *(u32*)(0x1FFFF7E8);
  Device_Serial1 = *(u32*)(0x1FFFF7EC);
  Device_Serial2 = *(u32*)(0x1FFFF7F0);

  Device_Serial0 += Device_Serial2;

  if (Device_Serial0 != 0)
  {
    IntToUnicode (Device_Serial0, &Virtual_Com_Port_StringSerial[2] , 8);
    IntToUnicode (Device_Serial1, &Virtual_Com_Port_StringSerial[18], 4);
  }
}

/*******************************************************************************
* Function Name  : HexToChar.
* Description    : Convert Hex 32Bits value into char.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
static void IntToUnicode (uint32_t value , uint8_t *pbuf , uint8_t len)
{
  uint8_t idx = 0;
  
  for( idx = 0 ; idx < len ; idx ++)
  {
    if( ((value >> 28)) < 0xA )
    {
      pbuf[ 2* idx] = (value >> 28) + '0';
    }
    else
    {
      pbuf[2* idx] = (value >> 28) + 'A' - 10; 
    }
    
    value = value << 4;
    
    pbuf[ 2* idx + 1] = 0;
  }
}


/*******************************************************************************
* Function Name  : Send DATA .
* Description    : send the data received from the AT32 to the PC through USB  
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
uint16_t CDC_Send_DATA (uint8_t *u8SendBuffer, uint16_t u16Sendlen)
{
  uint16_t  i = 0;
  if ( usb_txfifo.wrpointer >= USB_FIFO_MAX )
      usb_txfifo.wrpointer = 0;
    
  for (i = 0 ; usb_txfifo.wrpointer < USB_FIFO_MAX && i <  u16Sendlen; i ++ )
  {
      usb_txfifo.fifo[usb_txfifo.wrpointer] = u8SendBuffer[i];
      usb_txfifo.wrpointer ++;
      if ( usb_txfifo.wrpointer >= USB_FIFO_MAX )
          usb_txfifo.wrpointer = 0;  
  }
  usb_packet_sent = 1;
  return u16Sendlen;
}

/*******************************************************************************
* Function Name  : Receive DATA .
* Description    : receive the data from the PC to AT32 and send it through USB
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
uint16_t CDC_Receive_DATA(uint8_t *u8RcvBuffer, uint16_t u16RecvLen)
{ 
  uint16_t dwTmpLen = 0, i = 0, dwTmpPtr = usb_rxfifo.curpointer;
  if ( usb_rxfifo.curpointer == usb_rxfifo.wrpointer )
  {
      return 0;
  }
  if ( usb_rxfifo.wrpointer > usb_rxfifo.curpointer )
  {
      dwTmpLen = usb_rxfifo.wrpointer - usb_rxfifo.curpointer;
  }    
  else
  {
      dwTmpLen = USB_FIFO_MAX - usb_rxfifo.curpointer;
  }
  
  if ( dwTmpLen > u16RecvLen )
      dwTmpLen = u16RecvLen;
  
  usb_rxfifo.curpointer += dwTmpLen;
  if ( usb_rxfifo.curpointer >= USB_FIFO_MAX )
      usb_rxfifo.curpointer = 0;
  for ( i = 0; i < dwTmpLen; i ++ )
  {
      u8RcvBuffer[i] = usb_rxfifo.fifo[dwTmpPtr ++];
  }
  if ( usb_rxfifo.remaindpointer == 1 )
  {
      SetEPRxValid(ENDP3);
  }
  return dwTmpLen;
}

/*******************************************************************************
* Function Name  : USB_USART_SendData
* Description    : Save the Transmit data to Buffer.
* Input          : data: transmit data
* Output         : None.
* Return         : None.
*******************************************************************************/
void USB_USART_SendData(u8 data)
{
	usb_txfifo.fifo[usb_txfifo.wrpointer]=data;
	usb_txfifo.wrpointer++;
	if(usb_txfifo.wrpointer==USB_FIFO_MAX)//Overrun the Max_FIFO
	{
		usb_txfifo.wrpointer=0;
	} 
}

/*******************************************************************************
* Function Name  : usb_printf
* Description    : Printf data through USB to PC
* Input          : fmt: string
* Output         : None.
* Return         : None.
*******************************************************************************/
void usb_printf(char* fmt,...)  
{  
	u16 i,j;
	va_list ap;
	va_start(ap,fmt);
	vsprintf((char*)USART_PRINTF_Buffer,fmt,ap);
	va_end(ap);
	i=strlen((const char*)USART_PRINTF_Buffer);//Get the Length of the Transmit.
	for(j=0;j<i;j++)
	{
		USB_USART_SendData(USART_PRINTF_Buffer[j]); 
	}
} 



